;;
;; Software Breakpoints with INT3
;;

; RET: INDEX of BPX or -1 if error

RADbgSetBpx proc uses ebx esi edi p:ptr RADBGSTATE, VA:DWORD, HandlerProc:PBP_HANDLER_PROC

	assume ebx:ptr RADBGSTATE, esi:ptr RABPX

	mov ebx,p
	mov esi,[ebx].ps.pBpxBuff
	xor edi,edi
	.while [esi].VA 
		inc edi
		add esi,sizeof RABPX
		cmp edi,[ebx].ps.dwMaxBpx
		jae @@Err 
	.endw
	invoke RADbgReadMemB,ebx,VA
	mov [esi].OriginalByte,al
	invoke RADbgWriteMemB,ebx,VA,Int3
	m2mr [esi].VA,VA
	m2mr [esi].pHandler,HandlerProc
	mov eax,edi; Return index in buff
	ret
@@Err:	
	invoke RADbgLog,T('BPX BUFFER FULL')
	or eax,-1
	ret

	assume ebx:nothing, esi:nothing

RADbgSetBpx endp


RADbgClearBpx proc uses ebx esi edi p:ptr RADBGSTATE, Index:dword

	assume ebx:ptr RADBGSTATE, esi:ptr RABPX
	mov ebx,p
	mov edi,Index
	.if edi >= [ebx].ps.dwMaxBpx
		ret
	.endif
	imul edi,sizeof RABPX
	mov esi,[ebx].ps.pBpxBuff
	add esi,edi
	.if [esi].VA
		movzx eax,[esi].OriginalByte		
		invoke RADbgWriteMemB,ebx,[esi].VA,eax 
		invoke memset,esi,0,sizeof RABPX
	.endif
	ret
	assume ebx:nothing, esi:nothing

RADbgClearBpx endp

;;
;; Hardware Breakpoints with Debug Registers dr0 .. dr3
;;

SetThreadBpm proc uses edi esi ebx, put:PRADBGTHREAD, VA,Access,Len,HandlerProc
	assume ebx:ptr CONTEXT,esi:ptr RABPM
	mov eax,put
	test eax,eax
	jz @@Error
	lea esi,[eax].RADBGTHREAD.BpmBuff
	lea ebx,[eax].RADBGTHREAD.ctx	
	invoke GetCtx,put,CTX_DR
	mov edx,Len
	and edx,11b
	shl edx,4; shift length 2 bits left
	mov eax,Access
	and eax,11b
	add eax,edx;  and add Access
	.repeat
		xor ecx,ecx; ECX = 0
		xor edi,edi; DR 0
		.if [esi].VA == ecx; dr0 free?
			shl eax,DR7_RW0 ; len+rw 0
			mov ecx,VA; eax contains address
			or [ebx].Dr7_,mask DR7_L0; local bp in dr0
			mov [ebx].Dr0_,ecx
			.break
		.endif
		add esi,sizeof RABPM
		inc edi; DR 1
		.if [esi].VA == ecx; dr0 free?
			shl eax,DR7_RW1 ; len+rw 1
			mov ecx,VA; eax contains address
			or [ebx].Dr7_,mask DR7_L1; local bp in dr1
			mov [ebx].Dr1_,ecx
			.break
		.endif
		add esi,sizeof RABPM
		inc edi; DR 2
		.if [esi].VA == ecx; dr0 free?
			shl eax,DR7_RW2 ; len+rw 2
			mov ecx,VA; eax contains address
			or [ebx].Dr7_,mask DR7_L2; local bp in dr2
			mov [ebx].Dr2_,ecx
			.break
		.endif
		add esi,sizeof RABPM
		inc edi; DR 3
		.if [esi].VA == ecx; dr3 free?
			shl eax,DR7_RW3 ; len+rw 2
			mov ecx,VA; eax contains address
			or [ebx].Dr7_,mask DR7_L3; local bp in dr3
			mov [ebx].Dr2_,ecx
			.break
		.endif
	@@Error:
		; no free dr
		invoke RADbgLog,T('[ERROR] NO FREE DEBUG REGISTER OR INVALID THREAD')
		return -1  
	.until
	or [ebx].Dr7_,eax
	or [ebx].Dr7_,mask DR7_LE; Local Exact
	mov edx,HandlerProc
	mov al,byte ptr Len
	mov ah,byte ptr Access
	mov [esi].VA,ecx
	mov [esi].pHandler,edx
	mov [esi].Len,al; byte, word, dword
	mov [esi].Access,ah; read, read/write, execute
	invoke SetCtx,put
	return edi; 0..3
	assume esi:nothing,ebx:nothing
SetThreadBpm endp

ClearThreadBpm proc uses esi edi put:PRADBGTHREAD, drX:dword ; 0..3
	assume edi:ptr CONTEXT,esi:ptr RABPM
	mov eax,put
	test eax,eax
	jz @@Error
	lea esi,[eax].RADBGTHREAD.BpmBuff
	lea edi,[eax].RADBGTHREAD.ctx
	invoke GetCtx,put,CTX_DR
	mov ecx,drX
	imul eax,ecx,Sizeof RABPM; get offset in buff
	xor edx,edx
	add esi,eax
	xor eax,eax
	mov [esi].VA,edx; set to zero
	mov [esi].pHandler,edx;-||-
	mov [esi].Access,al;-||-
	mov [esi].Len,al;-||-
	.if ecx == edx; dr0?
		mov [edi].Dr0_,edx
		and [edi].Dr7_,not mask DR7_L0
	.endif    
	dec ecx
	.if zero?; dr1?
		mov [edi].Dr1_,edx
		and [edi].Dr7_,not mask DR7_L1
	.endif
	dec ecx
	.if zero?; dr2?
		mov [edi].Dr2_,edx
		and [edi].Dr7_,not mask DR7_L2
	.endif
	dec ecx
	.if zero?; dr3?
		mov [edi].Dr3_,edx
		and [edi].Dr7_,not mask DR7_L3
	.endif
	invoke SetCtx,put
@@Error:
	ret
	assume edi:nothing,esi:nothing
ClearThreadBpm endp

comment #
SetBpmEx proc p,VA,Access,Len,HandlerProc,dwTid
	invoke ThreadFromID,p,dwTid
	test eax,eax
	jz @@Error
	invoke SetThreadBpm,VA,Access,Len,HandlerProc,eax
	ret
@@Error:
	return -1
SetBpmEx endp

ClearBpmEx proc p,drX:dword,dwTid:dword; 0..3
	invoke ThreadFromID,p,dwTid
	test eax,eax
	jz @@Error
	invoke ClearThreadBpm,drX,eax
	ret
@@Error:
	return -1	
ClearBpmEx endp

SetBpm proc p,VA,Access,Len,HandlerProc
	invoke ThreadFromID,p,0
	test eax,eax
	jz @@Error
	invoke SetThreadBpm,VA,Access,Len,HandlerProc,eax
	ret
@@Error:
	return -1
SetBpm endp

ClearBpm proc p,drX:dword; 0..3
	invoke ThreadFromID,p,0
	test eax,eax
	jz @@Error
	invoke ClearThreadBpm,drX,eax
	ret
@@Error:
	return -1
ClearBpm endp
#