CTX_REGS equ CONTEXT_FULL
CTX_DR   equ CONTEXT_DEBUG_REGISTERS
CTX_FPU  equ CONTEXT_FLOATING_POINT

;Int1 EQU 0f1h
Int3 EQU 0cch
Noop EQU 090h

; EFLAGS

EFLAGS_TF equ (1 shl 8) ; Single-step mode active when set.
EFLAGS_IF equ (1 shl 9) 
EFLAGS_RF equ (1 shl 16); Flag to ensure single-step only happens once per instruction.

; DR7 bits

DEBUG_CONTROL RECORD \
	DR7_LEN3:2,DR7_RW3:2,\
	DR7_LEN2:2,DR7_RW2:2,\
	DR7_LEN1:2,DR7_RW1:2,\
	DR7_LEN0:2,DR7_RW0:2,\ ; length,read/write 
	DR7_Reserved1:2,     \
	DR7_GD:1,            \ ; general detect              
	DR7_Reserved2:3,     \
	DR7_GE:1,DR7_LE:1,   \ ; exact breakpoint
	DR7_G3:1,DR7_L3:1,   \ ; Global,Local
	DR7_G2:1,DR7_L2:1,   \ ; breakpoint enable
	DR7_G1:1,DR7_L1:1,   \  
	DR7_G0:1,DR7_L0:1

; DR6 bits

DEBUG_STATUS RECORD \
	DR6_Reserved1:16,   \
	DR6_BT:1,           \ ; task switch
	DR6_BS:1,           \ ; single step
	DR6_BD:1,           \ ; debug register access detected
	DR6_Reserved2:9,    \
	DR6_B3:1,           \
	DR6_B2:1,           \
	DR6_B1:1,           \ ; breakpoint condition detected
	DR6_B0:1

BPM_EX equ 00b;execute
BPM_WR equ 01b;write
BPM_IO equ 10b;i/o - undefined in user mode 
BPM_RW equ 11b;read/write
BPM_B equ 00b;byte
BPM_W equ 01b;word
BPM_D equ 11b;dword

;-- STATUS_BREAKPOINT CONTROL CODES ---
BP_SOURCE_DBGBREAK EQU 2
BP_SOURCE_ON_OEP   EQU 1
BP_SOURCE_NORMAL   EQU 0

;-- STATUS_SINGLE_STEP CONTROL CODES ---
SS_BP			equ 0; DR0..DR3
SS_TRACE		equ 1; EFLAGS_TF
SS_RESTORE_BPX	equ 2
SS_RESTORE_BPM	equ 3


DBGWAITTIMEOUT equ INFINITE
RADBG_VERBOSE equ 1

MAX_BPX equ 100
MAX_BPM equ 4 ;; 4 debug registers : dr0..dr3

BP_HANDLER_PROC TYPEDEF PROTO STDCALL :DWORD,:DWORD
PBP_HANDLER_PROC TYPEDEF PTR BP_HANDLER_PROC


RABPX struct ;; sizeof == 16
	VA dd ? ; virtual address
	pHandler PBP_HANDLER_PROC ? ; Handler procedure
	OriginalByte db ? ; Original Byte
	Reserved1 db 7 dup(?) ; align
RABPX ends
PRABPX typedef ptr RABPX

RABPM struct ;; sizeof == 16
	VA dd ? ; virtual address
	pHandler PBP_HANDLER_PROC ?
	Access db ? 
	Len db ?
	Reserved1 db 6 dup(?) ; align
RABPM ends
PRABPM typedef ptr RABPM

RADBGTHREAD struct
	pNext dd ?
	hThread dd ?
	dwThreadId dd ?
	StartAddress dd ?
	BpmBuff RABPM MAX_BPM DUP(<>)
	LastBpm RABPM <>	
	ctx CONTEXT <>
RADBGTHREAD ends
PRADBGTHREAD typedef ptr RADBGTHREAD

RADBGPROCESS struct
	hProcess dd ?
	dwProcessId dd ?
	dwThreadId dd ?
	hThread dd ?
	dwNumThreads dd ?
	pThreadList PRADBGTHREAD ?; thread list

	dwMaxBpx dd ?
	pBpxBuff PRABPX ?; breakpoint array
	LastBpx RABPX <>;
	
	dwBreakPointSource dd ?
	dwSingleStep dd ?
	EntryPointByte dd ?
		
RADBGPROCESS ends
PRADBGPROCESS typedef ptr RADBGPROCESS

RADBGPEIMAGE struct
	hFile dd ?
	dwFsize dd ?
	UNION
		pDos dd ?
		dwImageBase dd ?
	ENDS
	pNt dd ?
	pFirstSec dd ?
	SecNum dd ?
	SizeOfImage dd ?
	Entrypoint dd ?
RADBGPEIMAGE ends
PRADBGPEIMAGE typedef ptr RADBGPEIMAGE

RADBGSTATE struct

	pi PROCESS_INFORMATION <>
	de DEBUG_EVENT <>

	dwWaitTimeout DWORD  ?
	dwContinueStatus DWORD  ?
	dwOptions DWORD  ?

	pe RADBGPEIMAGE <>
	ps RADBGPROCESS <>

	hDbgThread DWORD ?
	dwDbgThreadID DWORD ?

	pszDebugeeFileName PBYTE ?;
	szDebugeePath BYTE MAX_PATH DUP(?)
	szCmdLn BYTE MAX_PATH DUP(?)

	sysinfo SYSTEM_INFO <>
	sinf STARTUPINFO <>

RADBGSTATE ends
PRADBGSTATE typedef ptr RADBGSTATE

